`timescale 1ns / 1ps
module seg7_cntrl(
    input          clk,
    input          rst,
    input    [3:0] din0,
    input    [3:0] din1,
    input    [3:0] din2,
    input    [3:0] din3,
    output   [3:0] an,
    output   [7:0] seg
);

// Generating ~kHz enable signal for FF-s -------------------------------------------------------------
reg [13:0] clk_div;
always @ (posedge clk)
if (rst)
    clk_div <= 0;
else
    clk_div <= clk_div + 1;

// The en signal is generated using an edge detector intead of a comparator
// There is only 1 positive edge on the cntr MSB in every cycle, it can be detected with a simple hw
// (Simpliet than a 13 bit comparator)

reg clk_div_13_dl; //Storing the privious state of the MSB
always @ (posedge clk)
    clk_div_13_dl <= clk_div[13];

reg en;
//Posedhge on MSB: previous state = 0, current state = 1 
always @ (posedge clk)
if (rst)
    en <= 0;
else
    en <= (clk_div[13] & ~clk_div_13_dl);

//Generating the anode signals (successive enable signals for each segment) --------------------------------
//The select input for the MUX will be a 2-bit counter synchronized to the anode signals
reg [3:0] an_shr = 4'b1110;
reg [1:0] an_cntr = 2'b00;
always @ (posedge clk)
	if (rst)
	begin
		an_shr  <= 4'b1110;
		an_cntr <= 0;
	end
	else if (en)
	begin
		an_shr  <= {an_shr[2:0], an_shr[3]};
		an_cntr <= an_cntr + 1;
	end

assign an = an_shr;

// 4:1 multiplexer for segment data -----------------------------------------------------------
reg [3:0] dmux;
always @ ( * )
case (an_cntr)
    0: dmux <= din0;
    1: dmux <= din1;
    2: dmux <= din2;
    3: dmux <= din3;
endcase

// Decoder for segment data -------------------------------------------------------------
reg [6:0] seg_dec;
always @(dmux)
case (dmux)
    4'b0000 : seg_dec <= 7'b1000000;   // 0
    4'b0001 : seg_dec <= 7'b1111001;   // 1
    4'b0010 : seg_dec <= 7'b0100100;   // 2
    4'b0011 : seg_dec <= 7'b0110000;   // 3
    4'b0100 : seg_dec <= 7'b0011001;   // 4
    4'b0101 : seg_dec <= 7'b0010010;   // 5
    4'b0110 : seg_dec <= 7'b0000010;   // 6
    4'b0111 : seg_dec <= 7'b1111000;   // 7
    4'b1000 : seg_dec <= 7'b0000000;   // 8
    4'b1001 : seg_dec <= 7'b0010000;   // 9
    4'b1101 : seg_dec <= 7'b0111111;   // - (13 is invalid for BCD - used for coding the sign)
    default  : seg_dec <= 7'b1111111;  // OFF for any other invalid BCD value
endcase

assign seg[6:0] = seg_dec;
assign seg[7] = an_shr[1]; //The decimal point for segment 3 is switched on, the others are off  

endmodule
